/*
******************************************************************************

   GGIART extension initialization.

   Author:	Eric Faurot <eric@ggi-project.org>
  
******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#include <ggi/internal/ggiart_int.h>

#include <ggi/gg.h>

/* Static variables 
 */
static int 	 _ggiartLibIsUp       = 0;
static void	*_ggiartConfigHandle  = NULL;
static char      _ggiartconfstub[512] = GGIARTCONFFILE;
static char     *_ggiartconffile      = _ggiartconfstub+GGIARTTAGLEN;

int _ggiartDebugSync = 0;
uint32 _ggiartDebugState = 0;

/* Extension ID. Defaulting to -1 should make segfault on abuse more likely.
 * This is exported so that sublibs can use it. */
EXPORTVAR ggi_extid _ggiartID = -1;

#define SYMPREFIX	"GGIARTdl_"


/* Dummy function which returns -1
 * We use this to reset the function pointers */
static int dummyfunc(void)
{
	GGIARTDPRINT_CORE("dummyfunc()\n");
	return -1;
}

/* Reset all function pointers to dummyfunc */
static void clearfuncs(ggiartpriv *priv)
{
  GGIARTDPRINT_CORE("clearfuncs()\n");

  priv->render_svp=dummyfunc;
  priv->render_buffer=dummyfunc;
  priv->fillscreen=dummyfunc;
}

/* This function is called by LibGGI when the API-list changes */
static int changed(ggi_visual_t vis, int whatchanged)
{
	GGIARTDPRINT_CORE("changed(%p, 0x%x)\n", vis, whatchanged);

	switch (whatchanged) 
	{
	    case GGI_CHG_APILIST: 
		{
			int i;
			const char *fname;
			ggi_dlhandle *lib;
			char api[GGI_MAX_APILEN], args[GGI_MAX_APILEN];
			
			clearfuncs(LIBGGI_GGIARTEXT(vis));

			for(i = 0; ggiGetAPI(vis, i, api, args) == 0; i++) 
			{
				strcat(api,"-art");
				fname = ggMatchConfig(_ggiartConfigHandle, api, NULL);
				if (fname == NULL) 
				{
					/* No special implementation for this sublib */
					continue;
				}
				lib = ggiExtensionLoadDL(vis, fname, args,
							NULL, SYMPREFIX);
			}
		}
		break;
	}
                                
	return 0;
}


/* Initialize the extension
 */
int ggiartInit(void)
{
        int err;
        char *str;
  
	str = getenv("GGIART_DEBUG");
	if (str != NULL) 
	{
		_ggiartDebugState = atoi(str);
		GGIARTDPRINT_CORE("Debugging=%d\n", _ggiartDebugState);
	}
	
	str = getenv("GGIART_DEBUGSYNC");
	if (str != NULL) 
	{
		_ggiartDebugSync = 1;
	}

	GGIARTDPRINT_CORE("ggiartInit()\n");
	
	_ggiartLibIsUp++;
	if (_ggiartLibIsUp > 1) 
	  return 0; /* Initialize only at first call. */
		
	err = ggLoadConfig(_ggiartconffile, &_ggiartConfigHandle);
	if (err != GGI_OK) 
	{
		fprintf(stderr,"LibGGIART: couldn't open %s\n", _ggiartconffile);
		_ggiartLibIsUp--;
		return err;
	}

	_ggiartID = ggiExtensionRegister("GGIART", sizeof(ggiartpriv), changed);

	if (_ggiartID < 0) 
	{
		fprintf(stderr, "LibGGIART: failed to register as extension.\n");
		_ggiartLibIsUp--;
		ggFreeConfig(_ggiartConfigHandle);
		return _ggiartID;
	}

	return 0;
}

/* Deinitialize the extension */
int ggiartExit(void)
{
	int rc;
    
	GGIARTDPRINT_CORE("ggiartExit()\n");
	
	if (!_ggiartLibIsUp) return -1;

	if (_ggiartLibIsUp > 1) 
	{
		/* Exit only at last call */
		_ggiartLibIsUp--;
		return 0;
	}
	
	rc = ggiExtensionUnregister(_ggiartID);
	ggFreeConfig(_ggiartConfigHandle);

	_ggiartLibIsUp = 0;

	return rc;
}


/* Attach the extension to a visual */
int ggiartAttach(ggi_visual_t vis)
{
	int rc;

	GGIARTDPRINT_CORE("ggiartAttach(%p) called\n", vis);

	rc = ggiExtensionAttach(vis, _ggiartID);
	if (rc == 0)
	{
		/* We are actually creating the primary instance. */
		memset(LIBGGI_GGIARTEXT(vis), 0, sizeof(ggiartpriv));
		/* Now fake an "API change" so the right libs get loaded */
		changed(vis, GGI_CHG_APILIST);
	}
	return rc;
}


/* Detach the extension from a visual */
int ggiartDetach(ggi_visual_t vis)
{
	GGIARTDPRINT_CORE("ggiartDetach(%p) called\n", vis);

	return ggiExtensionDetach(vis, _ggiartID);
}

/* This should be included by exactly _one_ file in each extension library. */
#include <ggi/internal/ggilibinit.h>
